' Main_Sub.bas 
' 1. 由 7個act_XXX()與2個scan_xxx()函式組成
' 2. 提供給 Main.bas 調用
'--------------------------------------------------------------------
'act_xxx()包含下面函式
' 1. actInit_main() 		: 系统初始化
' 2. actStop_main()			: 停止運行, 會使用子模塊引腳訊號，觸發子模塊運行
' 3. actHome_main()   		: 回原动作, 會使用子模塊引腳訊號，觸發子模塊運行
' 4. actPause_main()		: 系统暂停, 會使用子模塊引腳訊號，觸發子模塊運行
' 5. actResume_main() 		: 恢复暂停状态, 會使用子模塊引腳訊號，觸發子模塊運行
' 6. actErrorClear_main() 	: 清除错误
' 7. actFileSave_main()		: 文件保存
' 
'scan_xxx()包含下面函式
' 1. scanErr_main()  		: 扫描系统错误
' 2. scanIO_main()			: IO扫描

'---------------------------------------------------------------------------
' -----------------------------actXXX() 函数--------------------------------
'---------------------------------------------------------------------------

'***************************************
'1. actInit_S() : 系统初始化
'***************************************
'用到全局變量		: VR0 - VR99   VR1000 -VR1299
'用到子模塊引腳		: 無
Function  actInit_main() as i16
	
	'Step 1 ：初始化VR表
	VRRESET(0,99)						'复位VR,复位VR0~VR150
	FILE_READVR "ExampleParameters.csv"		'加载配置参数VR300~VR399
	'......								'添加您需要初始化的配置到此
	
	'Step 2 ：启动其他Task
	Run_Task "Table"
	return 1
End Function

'***************************************
'2. actStop_main() : 停止運行
'***************************************
'用到全局變量		:SysMode
'用到子模塊引腳		:SetStop_TB
Function actStop_main()	AS I16
	
	IF SysMode = SysMode_Pause 	THEN SysMode = BePause_main			'讀回之前暂停前的系統模式
	IF SysMode = SysMode_Home 	THEN SysMode = SysMode_Init			'如果在Home中,则判断为Home失败，系统回到初始化状态
	IF SysMode = SysMode_Auto 	THEN SysMode = SysMode_Ready		'如果在AutoRun中,则停止后退出AutoRun流程
	
	HomeStep = 0
	SetStop_TB = 1
	return 1
End Function 

'***************************************
'3. actPause_main() : 系统暂停
'***************************************
'暂停机制			: SetPause_TB=1 会触发 Table 模块暂停 (其他模块也是相同方式)
'用到全局變量		: SysMode, SysMode_Pause
'用到子模塊引腳		: SetPause_TB
Function actPause_main() as I16
	'当设备处于<暂停> <错误> <Home>模式时，不对暂停进行处理
	IF SysMode <> SysMode_Pause AND SysMode<>SysMode_Err AND SysMode<>SysMode_Home Then 		
		BePause_main		= SysMode 								'记录暂停前的设备状态
		SysMode 			= SysMode_Pause							'将系统设置为暂停模式
		SetPause_TB 		= 1
	END IF 
	return 1
End Function 

'***************************************
'4. actResume_main() : 恢复暂停状态
'***************************************
'恢复机制			:SetResume_TB=1 会触发Table模块解除暂停状态(其他模块也是相同方式)
'用到全局變量		:SysMode
'用到子模塊引腳		:SetResume_TB
Function actResume_main() as i16
	SysMode					= BePause_main							'将暂停前的系统模式恢复到当前
	SetResume_TB 			= 1
	return 1
End Function 

'***************************************
'5. actHome_main()   : 回原动作
'***************************************
'用到全局變量		:HomeStep, HomeFlag
'用到子模塊引腳		:SetHome_TB
'Home()的流程步骤
Function actHome_main()  as i16                       				'回原动作
	Select Case Cint(HomeStep)
		Case 0 											
			HomeFlag 	= 0											'将Home完成旗标设置为0
			HomeStep	= 10
		
		Case 10
			If MS_PULSE(HomeStep) 		Then SetHome_TB=1			'MS_PULSE指令检测第一次进入此Case。设置左工位Home信号有效,左工位会自动进入Home流程
			If SetHome_TB  	= 0 		Then HomeStep =StepDone		'如果标志清0表示Home动作已经完成，进入下一个Home流程
			
		Case StepDone
			print "System Home Done !"
			HomeFlag		= 1										'原点动作完成，设置系统原点旗标为1
			HomeStep 		= 0
			return 1
			
	End Select 
	return 0
End Function 

'***************************************
'6. actErrorClear_main() : 清除错误
'***************************************
'用到全局變量		:無
'用到子模塊引腳		:無
Function actErrorClear_main() as i16        
    BASE X1,Y1,Z1
	RESETERR  
	CLEAR_ERROR
	
	SysErrMsg = 0 
	return 1
END Function 

'***************************************
'7. actFileSave_main() : 文件保存
'***************************************
'用到子模塊引腳		:無
Function actFileSave_main() as i16
	FILE_WRITEVR "ExampleParameters.csv",1000,1299 				'保存参数到指定文件内
	print "Save File Success !"
	return 1	
End Function 


'---------------------------------------------------------------------------
'--------------------------------scanXXX 函式-------------------------------
'---------------------------------------------------------------------------

'***************************************
'1. scanErr_main() : 扫描系统错误
'***************************************
'用到子模塊引腳：無
Sub scanErr_main()  
	
	'1,通过扫描轴MIO来判断轴是否在错误状态
	
	'a,检查轴错误
	IF ERROR_AXIS <> 0 and SysMode <> SysMode_Err Then 			'当轴存在Err,则扫描具体的错误信息
		For i as I16 = 0 to 3									'扫描轴0到轴5(请根据实际板卡轴数上限修改)
			BASE i
			IF STATE = 0 or STATE = 3 Then 
				IF MIO.ALM 	= 1	Then	SysErrMsg = 1			'通过MIO回读是否出现轴报警
				IF MIO.PEL 	= 1 Then	SysErrMsg = 2			'通过MIO回读是否出现轴正极限
				IF MIO.NEL 	= 1 Then 	SysErrMsg = 3			'通过MIO回读是否出现轴负极限
				IF MIO.SPEL = 1	Then 	SysErrMsg = 4			'通过MIO回读是否出现轴软正极限
				IF MIO.SNEL = 1 Then	SysErrMsg = 5			'通过MIO回读是否出现轴软负极限
				IF MIO.EMG 	= 1 Then 	SysErrMsg = 6			'通过MIO回读是否出现轴急停
			End IF 
		Next i
	End IF 
	
	'b,检查EtherCat通信错误
	IF SysMode <> SysMode_Err  	and CTLSTATUS <> 0 	Then 		'检测EtherCat通信是否异常，脉冲卡可以删除此行
			IF CTLSTATUS.MRDIS 	Then 	SysErrMsg = 7			'检测是否有MotionRing掉线
			IF CTLSTATUS.IORDIS Then 	SysErrMsg = 8			'检测是否有IORing掉线
	End IF 
	
	'c,自行添加其他错误扫描到此处
	'....
	
	IF SysErrMsg<>0 		Then 	SysMode = SysMode_Err		'有错误时,系统进入错误模式
	
End Sub

'***************************************
'2. scanIO_main(): IO扫描
'***************************************
'用到子模塊引腳：無
Sub scanIO_main()							'扫描IO
	IF MS_EDGER(DIN(10)) THEN DOUT(23) = 1		'真空吸附按钮触发时，打开真空泵
	IF MS_EDGER(DIN(5)) THEN DOUT(12) = 1		'环形光源按钮触发时，打开环形光源
	'....添加其他IO扫描到此处
			
End Sub 
