' Table_Sub.bas 
' 1. Table模塊的子函式
' 2. 由 5個基本 act_XXX()與3個手動act_XXX()函式組成
' 3. 函式只提供給 Table.bas 與 Table_AutoRun.bas 調用，函式間無互相調用關係
'--------------------------------------------------------------------
' 包含下列 Function, 所有的actXXX() 其回傳值： (0)表示動作未完成  (1)表示動作完成
' 基本Function：
' 1. actInit_TB()		'初始化參數
' 2. actStop_TB()		'Table模塊的停止運行
' 3. actPause_TB()		'Table模塊的暫停運行
' 4. actResume_TB()		'Table模塊的恢复運行
' 5. actHome_TB()		'Table模塊的回原点
'


'------------------------------------------------------------------------
'--------------------------------基本函数--------------------------------
'------------------------------------------------------------------------

'***************************************
'1. actInit_TB() : 初始化參數
'***************************************
Function actInit_TB() AS I16
	'先复位使用的VR表
	VRRESET(100,99)
	
	'复位VR1000 - VR1009 
	'VRRESET指令會將VR表工具中的初始值設定會載入VR
	VRRESET(1000, 10)
	
	'初始化轴配置
	BASE X1,Y1,Z1
	SVON 
	UNIT_DENOM = 1
	UNIT_NUM =1000
	ALM_EN = 1
	SPEL_EN = 1
	SNEL_EN = 1
	SPEL =1000*UNIT_NUM/UNIT_DENOM
	SNEL =-100*UNIT_NUM/UNIT_DENOM
	JK = 1
	ACC = 500
	DEC =500
	VL = 20
	VH = 100
	
	'初始化模块引脚为0
	SetInit_TB		= 0 
	SetHome_TB		= 0	
	SetPause_TB		= 0		
	SetResume_TB	= 0		
	SetStop_TB		= 0			
	SetAuto_TB		= 0	
	AutoStep_TB 	= 0 
	TBStep 			= 0
	return 1
END Function


'***************************************
'2. actStop_TB() : 初始化參數
'***************************************
Function actStop_TB() AS I16
	'停止轴运动
	base X1,Y1,Z1
	STOPDEC 
	
	'将所有Table模块引脚都设置为0
	TBStep 	= 0
	AutoStep_TB 	= 0 
	SetStop_TB 		= 0
	SetPause_TB 	= 0 
	SetResume_TB	= 0 
	SetHome_TB 		= 0 
	SetAuto_TB		= 0 
	
	return 1
End Function 

'***************************************
' 3. actPause_TB(): Table模塊的暫停運行
'***************************************
'描述：會先執行停止後，將目前的状态/步驟記錄下來，供恢復時使用
'回傳值： (0)未完成  (1)完成動作
'用到的全局變量： 无
Function actPause_TB() AS I16
	base X1,Y1,Z1							
	stopdec														'立即停止轴运动
	return 1
End Function 

'***************************************
' 4. actResume_TB(): 'Table模塊的恢复運行
'***************************************
'描述：會恢复之前記錄的状态/步驟
'回傳值： (0)未完成  (1)完成動作
'用到的全局變量： BePause_TB, TBMode
Function actResume_TB() AS I16
	TBMode 			= BePause_TB								'还原暂停前的状态
	return 1
End Function 

'***************************************
' 5. actHome_TB(): Table模塊的回原点
'***************************************
'描述：動作分多個步驟，由TBStep控制步驟的運行, 由case 0 開始, Case StepDone 結束
'回傳值： (0)未完成  (1)完成動作
'用到的全局變量： TBStep, WaitPosLeft
' !!注意！！范例中模拟使用MOVEABS，实际运行时，请根据机构类型改为HomeP/HomeN
Function actHome_TB() AS I16
	Select case Cint(TBStep)
		case 0													'先设置轴Home所需要用到的参数
			base X1,Y1,Z1
			SPEL_EN = 0
			SNEL_EN =0 
			HOME_MODE = 0
			HOME_ACC = 100
			HOME_DEC = 100
			HOME_VL = 2
			HOME_VH = 10
			TBStep = 10
			
		case 10
			MOVEABS AX(Z1),0									'范例中并没有下Home指令，实际运行时根据机构类型改为HomeP或者HomeN
			TBStep = 20											'Home和相对运动时，不能在本case等待运动到位
		
		case 20													'等待Z轴Home到位
			IF AxIsReady(Z1) THEN TBStep = 30
			
		case 30 
			MOVEABS AX(X1),0									'范例中并没有下Home指令，实际运行时根据机构类型改为HomeP或者HomeN
			TBStep = 40											'Home和相对运动时，不能在本case等待运动到位
		
		case 40
			IF AxIsReady(X1) THEN TBStep = 50
			
		case 50
			MOVEABS AX(Y1),0									'范例中并没有下Home指令，实际运行时根据机构类型改为HomeP或者HomeN
			TBStep = 60											'Home和相对运动时，不能在本case等待运动到位
		
		case 60
			IF AxIsReady(Y1) THEN TBStep = StepDone
		
			
		case StepDone											'原点动作结束,复位引脚,复位步骤,返回结果
			BASE X1,Y1,Z1
			SPEL_EN = 1
			SNEL_EN = 1
			TBStep 	= 0 
			return 1
		
	end select 
	return 0
End Function  

'------------------------------------------------------------------------
'--------------------------------手动函数--------------------------------
'------------------------------------------------------------------------
'负责实现机头到等待位置.   为了动作不会撞机, 我们先让Z轴往上移动, 再移动XY轴.
'使用的变量如下(未说明的变量都是框架预设的变量)
'WaitPosZ: Z1轴等待位置 (请在 ProjectDefine.bi 宣告)
'WaitPosX, WaitPosY: X1, Y1轴等待位置 (请在 ProjectDefine.bi 宣告)
Function actWaitPos_TB()AS I16
	Select Case Cint(TBStep)
	case 0
		IF AXISREADY(Z1) THEN MOVEABS AX(Z1),WaitPosZ	
		IF AXISREADY(Z1) THEN TBStep= 10
	case 10 
		 BASE X1, Y1
		IF AXISREADY(X1,Y1) THEN MOVEABS WaitPosX, WaitPosY	
		IF AXISREADY(X1,Y1) THEN TBStep= StepDone
	case StepDone 
		TBStep = 0 
		Return 1
	End Select 
	return 0 
End Function

'负责实现机头到取螺丝位置, 并取螺丝.
'使用的变量如下, 请在ProjectDefine.bi 宣告 (未说明的变量都是框架预设的变量)
'GetScrewPosX, GetScrewPosY : VR变量, X1, Y1轴取螺丝位置
'GetScrewPosZ : VR变量, Z1轴取螺丝位置
'T1: TMR对象变量, 延迟计数器
'WaitPosZ : VR变量, Z1轴等待位置
Function actGetScrew_TB() AS I16
	Select Case Cint(TBStep)
	case 0		'X1, Y1轴移动到取螺丝位置
		BASE X1, Y1
		IF AXISREADY(X1,Y1) THEN MOVEABS GetScrewPosX, GetScrewPosY
		IF AXISREADY(X1,Y1) THEN TBStep = 10
	case 10 	'Z1下降到取螺丝位置
		IF AXISREADY(Z1) THEN MOVEABS AX(Z1),GetScrewPosZ
		IF AXISREADY(Z1) THEN TBStep = 20
		case 20	'机头取螺丝, 用DOUT0控制汽缸, 使用T1延时, 并先重置T1
		DOUT(0)=1
		T1.Reset()
		TBStep = 30
	case 30	'延迟0.5秒
		IF T1.On(500) THEN TBStep = 40
	case 40 '机头Z1上升
		IF AXISREADY(Z1) THEN MOVEABS AX(Z1),WaitPosZ	
		IF AXISREADY(Z1) THEN TBStep = StepDone 
	case StepDone 	'完成, 将 TBStep 重置0, 并回传1
		TBStep = 0 
		Return 1
	End Select 
	return 0 
End Function 


P_Array(0)=Pt(150,150)        '位置点 P0 
P_Array(1)=Pt(250,150)       '位置点 P1
P_Array(2)=Pt(250,250)       '位置点 P2 
P_Array(3)=Pt(150,250)        '位置点 P3 

'负责实现机头将取得的螺丝锁付到工件上. 这个动作可分为5个小动作,
' 1. 机头X1,Y1移动到工件锁螺丝位置上方
' 2. 机头Z1下降到启动位置
' 3. 机头Z1锁螺丝(DO1)动作, 并下降锁付长度
' 4. 关闭DO1
' 5. 机头Z1上升到等待位置
'使用的变量如下, 请在 ProjectDefine.bi 宣告(未说明的变量都是框架预设的变量)
' StartScrewPosZ: VR变量, Z1轴准备开始锁螺丝位置
' ScrewLen: VR变量, Z1轴锁螺丝的深度
' WaitPosZ : VR变量, Z1轴等待位置
' P_Array(4): PT变量, 锁付螺丝4个点的位置
Function actDoScrew_TB() AS I16
	Select Case Cint(TBStep)
	case 0	'X1,Y1直线差补移动到P1点上方
		BASE X1,Y1
		IF AXISREADY(X1,Y1) THEN LINEABS P_Array(ScrewNum)
		IF AXISREADY(X1,Y1) THEN TBStep= 10
	case 10 'Z1下降到启动位置
		IF AXISREADY(Z1) THEN MOVEABS AX(Z1), StartScrewPosZ	
		IF AXISREADY(Z1) THEN TBStep= 20
	case 20 '锁螺丝(DO1)动作, 并机头Z1下降锁付长度
		DOUT(1)=1
		IF AXISREADY(Z1) THEN MOVEABS AX(Z1), StartScrewPosZ+ScrewLen
		IF AXISREADY(Z1) THEN TBStep= 30
	case 30 '关闭DO1(停止螺丝披头), 并切换到下一步骤
		DOUT(1)=0 
		TBStep = 40
	case 40 '机头Z1上升到等待位置
		IF AXISREADY(Z1) THEN MOVEABS AX(Z1),WaitPosZ	
		IF AXISREADY(Z1) THEN TBStep= StepDone 
	case StepDone '完成, 将 TBStep重置0, 并回传1代表结束!
		TBStep= 0 
		Return 1
	End Select 
	return 0 
End Function 



